'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNetworkInterface = createNetworkInterface;

var _ramda = require('ramda');

var _networkInterface = require('apollo-client/transport/networkInterface');

var _phoenix = require('phoenix');

var applyWares = function applyWares(ctx, wares) {
  return new Promise(function (resolve, reject) {
    var queue = function queue(funcs, scope) {
      var next = function next() {
        if (funcs.length > 0) {
          var f = funcs.shift();
          f && (f.applyMiddleware || f.applyAfterware).apply(scope, [ctx, next]);
        } else {
          resolve(ctx);
        }
      };
      next();
    };
    queue(wares.slice());
  });
};

var createSocket = function createSocket(options) {
  var uri = options.uri;

  var Socket = options.Socket || _phoenix.Socket;

  if (typeof Socket.constructor === 'function') {
    return new Socket(uri, options);
  } else {
    return Socket(uri, options);
  }
};

var getSocket = function getSocket(sockets, options) {
  if (options.logger === true) {
    options.logger = function (kind, msg, data) {
      return console.log('Apollo websocket ' + kind + ': ' + msg, data);
    };
  }

  var uri = options.uri;

  if (!uri) throw 'Missing apollo websocket options.uri';

  if (sockets[uri]) {
    return sockets[uri];
  }

  return sockets[uri] = {
    conn: createSocket(options),
    channels: {}
  };
};

var getChannel = function getChannel(socket, options) {
  var channel = options.channel;


  if (!channel) throw "Missing options.channel";
  if (!channel.topic) throw "Missing options.channel.topic";

  if (socket.channels[channel.topic]) {
    return socket.channels[channel.topic];
  } else {
    return socket.channels[channel.topic] = {
      conn: socket.conn.channel(channel.topic, channel.params || {}),
      queue: []
    };
  }
};

var socketConnect = function socketConnect(sockets, options) {
  return new Promise(function (resolve, reject) {
    var socket = getSocket(sockets, options);
    if (socket.conn.isConnected()) {
      resolve(socket);
    } else {
      socket.conn.onOpen(function (_) {
        return resolve(socket);
      });
      socket.conn.onError(reject);
      socket.conn.connect();
    }
  });
};

var channelJoin = function channelJoin(socket, options) {
  return new Promise(function (resolve, reject) {
    var channel = getChannel(socket, options);
    if (channel.conn.joinedOnce) {
      resolve(channel);
    } else if (!channel.conn.isJoining()) {
      channel.conn.join().receive('ok', function (_) {
        return resolve(channel);
      }).receive('error', reject).receive('timeout', reject);
    }
  });
};

var sendQuery = function sendQuery(channel, context) {
  return new Promise(function (resolve, reject) {
    var options = context.options,
        request = context.request;


    var msg = options.channel.in_msg || 'gql';
    var payload = (0, _networkInterface.printRequest)(request);

    channel.conn.push(msg, payload).receive('ok', resolve).receive('error', reject).receive('timeout', reject);
  });
};

function executeQuery(sockets, context) {
  return new Promise(function (resolve, reject) {
    var request = context.request,
        options = context.options;


    var queryPromise = (0, _ramda.pipeP)(function (_) {
      return socketConnect(sockets, options);
    }, function (socket) {
      return channelJoin(socket, options);
    }, function (channel) {
      return sendQuery(channel, context);
    });

    // let middleware handle any error
    return queryPromise().then(resolve, resolve);
  });
}

var isEmptyOrNil = (0, _ramda.either)(_ramda.isEmpty, _ramda.isNil);

var responseData = function responseData(_ref) {
  var response = _ref.response;

  return new Promise(function (resolve, reject) {
    if (isEmptyOrNil(response)) {
      reject('No response');
    } else if (!isEmptyOrNil(response.data)) {
      resolve(response);
    } else {
      reject(response);
    }
  });
};

function createNetworkInterface(ifaceOpts) {
  var sockets = {};
  var middlewares = [];
  var afterwares = [];

  var use = (0, _ramda.map)(function (item) {
    return middlewares.push(item);
  });
  var useAfter = (0, _ramda.map)(function (item) {
    return afterwares.push(item);
  });

  var requestMiddleware = function requestMiddleware(request) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ request: request, options: options }, middlewares);
  };

  var responseMiddleware = function responseMiddleware(response) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ response: response, options: options }, afterwares);
  };

  var query = (0, _ramda.pipeP)(requestMiddleware, executeQuery.bind(null, sockets), responseMiddleware, responseData);

  return { query: query, use: use, useAfter: useAfter };
}